==> modonomicon-1.18.2-1.3.1-sources.jar: com/klikli_dev/modonomicon/multiblock/matcher/BlockStateMatcher.java <==
/*
 * SPDX-FileCopyrightText: 2022 klikli-dev
 *
 * SPDX-License-Identifier: MIT
 */

package com.klikli_dev.modonomicon.multiblock.matcher;

import com.google.gson.JsonObject;
import com.klikli_dev.modonomicon.Modonomicon;
import com.klikli_dev.modonomicon.api.multiblock.StateMatcher;
import com.klikli_dev.modonomicon.api.multiblock.TriPredicate;
import com.mojang.brigadier.StringReader;
import com.mojang.brigadier.exceptions.CommandSyntaxException;
import net.minecraft.commands.arguments.blocks.BlockStateParser;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Registry;
import net.minecraft.network.FriendlyByteBuf;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.util.GsonHelper;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.block.state.BlockState;

import java.util.Objects;

/**
 * Matches a BlockState, respecting all BlockState properties.
 */
public class BlockStateMatcher implements StateMatcher {
    public static final ResourceLocation TYPE = Modonomicon.loc("blockstate");

    private final BlockState displayState;
    private final BlockState blockState;
    private final TriPredicate<BlockGetter, BlockPos, BlockState> predicate;

    protected BlockStateMatcher(BlockState displayState, BlockState blockState) {
        this.displayState = displayState;
        this.blockState = blockState;
        this.predicate = (blockGetter, blockPos, state) -> state == blockState;
    }

    public static BlockStateMatcher from(BlockState blockState) {
        return new BlockStateMatcher(null, blockState);
    }

    public static BlockStateMatcher from(BlockState displayState, BlockState blockState) {
        return new BlockStateMatcher(displayState, blockState);
    }

    public static BlockStateMatcher fromJson(JsonObject json) {
        BlockState displayState = null;
        if (json.has("display")) {
            try {
                displayState = new BlockStateParser(new StringReader(GsonHelper.getAsString(json, "display")), false).parse(false).getState();
            } catch (CommandSyntaxException e) {
                throw new IllegalArgumentException("Failed to parse BlockState from json member \"display\" for BlockStateMatcher.", e);
            }
        }

        try {
            var blockState = new BlockStateParser(new StringReader(GsonHelper.getAsString(json, "block")), false).parse(false).getState();
            return new BlockStateMatcher(displayState, blockState);
        } catch (CommandSyntaxException e) {
            throw new IllegalArgumentException("Failed to parse BlockState from json member \"block\" for BlockStateMatcher.", e);
        }

    }

    public static BlockStateMatcher fromNetwork(FriendlyByteBuf buffer) {
        try {
            BlockState displayState = null;
            if (buffer.readBoolean())
                displayState = new BlockStateParser(new StringReader(buffer.readUtf()), false).parse(false).getState();

            var blockState = new BlockStateParser(new StringReader(buffer.readUtf()), false).parse(false).getState();

            return new BlockStateMatcher(displayState, blockState);
        } catch (CommandSyntaxException e) {
            throw new IllegalArgumentException("Failed to parse BlockStateMatcher from network.", e);
        }
    }

    @Override
    public ResourceLocation getType() {
        return TYPE;
    }

    @Override
    public BlockState getDisplayedState(long ticks) {
        return this.displayState == null ? this.blockState : this.displayState;
    }

    @Override
    public TriPredicate<BlockGetter, BlockPos, BlockState> getStatePredicate() {
        return this.predicate;
    }

    @Override
    public void toNetwork(FriendlyByteBuf buffer) {
        buffer.writeBoolean(this.displayState != null);
        if (this.displayState != null)
            buffer.writeUtf(BlockStateParser.serialize(this.displayState));
        buffer.writeUtf(BlockStateParser.serialize(this.blockState));
    }

    @Override
    public int hashCode() {
        return Objects.hash(this.blockState, this.displayState);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || this.getClass() != o.getClass()) {
            return false;
        }
        var that = (BlockStateMatcher) o;
        return this.blockState.equals(that.blockState) && this.displayState.equals(that.displayState);
    }
}

==> modonomicon-1.19.2-1.21.0-sources.jar: com/klikli_dev/modonomicon/multiblock/matcher/BlockStateMatcher.java <==
/*
 * SPDX-FileCopyrightText: 2022 klikli-dev
 *
 * SPDX-License-Identifier: MIT
 */

package com.klikli_dev.modonomicon.multiblock.matcher;

import com.google.gson.JsonObject;
import com.klikli_dev.modonomicon.Modonomicon;
import com.klikli_dev.modonomicon.api.multiblock.StateMatcher;
import com.klikli_dev.modonomicon.api.multiblock.TriPredicate;
import com.mojang.brigadier.StringReader;
import com.mojang.brigadier.exceptions.CommandSyntaxException;
import net.minecraft.commands.arguments.blocks.BlockStateParser;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Registry;
import net.minecraft.network.FriendlyByteBuf;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.util.GsonHelper;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.block.state.BlockState;

import java.util.Objects;

/**
 * Matches a BlockState, respecting all BlockState properties.
 */
public class BlockStateMatcher implements StateMatcher {
    public static final ResourceLocation TYPE = Modonomicon.loc("blockstate");

    private final BlockState displayState;
    private final BlockState blockState;
    private final TriPredicate<BlockGetter, BlockPos, BlockState> predicate;

    protected BlockStateMatcher(BlockState displayState, BlockState blockState) {
        this.displayState = displayState;
        this.blockState = blockState;
        this.predicate = (blockGetter, blockPos, state) -> state == blockState;
    }

    public static BlockStateMatcher from(BlockState blockState) {
        return new BlockStateMatcher(null, blockState);
    }

    public static BlockStateMatcher from(BlockState displayState, BlockState blockState) {
        return new BlockStateMatcher(displayState, blockState);
    }

    public static BlockStateMatcher fromJson(JsonObject json) {
        BlockState displayState = null;
        if (json.has("display")) {
            try {
                displayState = BlockStateParser.parseForBlock(Registry.BLOCK, new StringReader(GsonHelper.getAsString(json, "display")), false).blockState();
            } catch (CommandSyntaxException e) {
                throw new IllegalArgumentException("Failed to parse BlockState from json member \"display\" for BlockStateMatcher.", e);
            }
        }

        try {
            var blockState = BlockStateParser.parseForBlock(Registry.BLOCK, new StringReader(GsonHelper.getAsString(json, "block")), false).blockState();
            return new BlockStateMatcher(displayState, blockState);
        } catch (CommandSyntaxException e) {
            throw new IllegalArgumentException("Failed to parse BlockState from json member \"block\" for BlockStateMatcher.", e);
        }

    }

    public static BlockStateMatcher fromNetwork(FriendlyByteBuf buffer) {
        try {
            BlockState displayState = null;
            if (buffer.readBoolean())
                displayState = BlockStateParser.parseForBlock(Registry.BLOCK, new StringReader(buffer.readUtf()), false).blockState();

            var blockState = BlockStateParser.parseForBlock(Registry.BLOCK, new StringReader(buffer.readUtf()), false).blockState();

            return new BlockStateMatcher(displayState, blockState);
        } catch (CommandSyntaxException e) {
            throw new IllegalArgumentException("Failed to parse BlockStateMatcher from network.", e);
        }
    }

    @Override
    public ResourceLocation getType() {
        return TYPE;
    }

    @Override
    public BlockState getDisplayedState(long ticks) {
        return this.displayState == null ? this.blockState : this.displayState;
    }

    @Override
    public TriPredicate<BlockGetter, BlockPos, BlockState> getStatePredicate() {
        return this.predicate;
    }

    @Override
    public void toNetwork(FriendlyByteBuf buffer) {
        buffer.writeBoolean(this.displayState != null);
        if (this.displayState != null)
            buffer.writeUtf(BlockStateParser.serialize(this.displayState));
        buffer.writeUtf(BlockStateParser.serialize(this.blockState));
    }

    @Override
    public int hashCode() {
        return Objects.hash(this.blockState, this.displayState);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || this.getClass() != o.getClass()) {
            return false;
        }
        var that = (BlockStateMatcher) o;
        return this.blockState.equals(that.blockState) && this.displayState.equals(that.displayState);
    }
}
